// Copyright (C) 2011 Stephen Warren
// Licence: GPLv2+
//
// Based on Panel Favorites Copyright (C) 2011 R M Yorston
// Based on QuickLaunch@github.com

const FileUtils = imports.misc.fileUtils;
const Gio = imports.gi.Gio;
const Gtk = imports.gi.Gtk;
const GLib = imports.gi.GLib;
const Lang = imports.lang;
const Main = imports.ui.main;
const St = imports.gi.St;
const PanelMenu = imports.ui.panelMenu;
const Meta = imports.gi.Meta;

const PopupMenu = imports.ui.popupMenu;

// FIXME: Allow this to be configured in gconf?
const qlPath = GLib.get_home_dir() + '/.local/user/quicklaunch';

function fullListDirAsync(file, callback) {
    let allFiles = [];
    file.enumerate_children_async("standard::*", //Gio.FILE_ATTRIBUTE_STANDARD_NAME,
                                  Gio.FileQueryInfoFlags.NONE,
                                  GLib.PRIORITY_LOW, null, function (obj, res) {
        let enumerator = obj.enumerate_children_finish(res);
        function onNextFileComplete(obj, res) {
            let files = obj.next_files_finish(res);
            if (files.length) {
                allFiles = allFiles.concat(files);
                enumerator.next_files_async(100, GLib.PRIORITY_LOW, null, onNextFileComplete);
            } else {
                enumerator.close(null);
                callback(allFiles);
            }
        }
        enumerator.next_files_async(100, GLib.PRIORITY_LOW, null, onNextFileComplete);
    });
}

function PopupMenuItem(label, icon, callback) {
    this._init(label, icon, callback);
}

PopupMenuItem.prototype = {
    __proto__: PopupMenu.PopupBaseMenuItem.prototype,

    _init: function(text, icon, callback) {
        PopupMenu.PopupBaseMenuItem.prototype._init.call(this);

        //this.icon = new St.Icon({ icon_name: icon,
        this.icon = new St.Icon({ gicon: icon,
                                  icon_size: 32,
                                  icon_type: St.IconType.FULLCOLOR,
                                  style_class: 'popup-menu-icon' });
        this.icon.set_size(32,32);
        this.addActor(this.icon);
        this.label = new St.Label({ text: text });
        this.addActor(this.label);
        this.connect('activate',  callback);
    }
};

function QLButton(app) {
    this._init(app);
}

QLButton.prototype = {
    __proto__ : PanelMenu.SystemStatusButton.prototype,

    _init: function(appinfo) {

        this.appinfo = appinfo;

        PanelMenu.SystemStatusButton.prototype._init.call(this,appinfo.get_icon().to_string());
        if(appinfo.get_icon().to_string().indexOf("symbolic") == -1 ) {
            this._iconActor.icon_type = 1;
        }
        this.actor._delegate = this;

        let cmdLine = appinfo.get_commandline().split(' ');
        if(cmdLine && 
            Gio.file_new_for_path(cmdLine[cmdLine.length-1]).query_file_type(Gio.FileQueryInfoFlags.NONE,null) == Gio.FileType.DIRECTORY) {
            this.dirname = cmdLine[cmdLine.length-1]
            this.menu.open = Lang.bind(this,this.generateMenu)
            this.menu.actor.add_style_class_name("master");
        }
        else {
            this.menu.open = Lang.bind(this,function() {this.appinfo.launch([], null);});
        }
    },

    _focusDesktopActor: function() {
        let windows = global.get_window_actors().filter(function(w) {
            return w.meta_window.get_window_type() == Meta.WindowType.DESKTOP;
        });
        if (windows.length == 0)
            return;
        global.display.set_input_focus_window(windows[0].meta_window,false,0);
    },

    generateMenu: function(animate) {
        if( global.get_pointer()[2] == 512 ) {
            this._focusDesktopActor();
            this.appinfo.launch([], null);
            return;
        }
        this.menu.removeAll();
        let dir = Gio.file_new_for_path(this.dirname);
        this.submenu = new PopupMenu.PopupSubMenuMenuItem(dir.get_basename());
        this.menu.addMenuItem(this.submenu);

        let fileEnum;
        let file, info;
        try {
            fullListDirAsync(dir,Lang.bind(this,function(filelist) {
                let skipReg = /^\.|~$/;
                let regIcon = /(Icon=)(.+)$/m;
                filelist = filelist.sort(function(a,b) { return a.get_file_type()>b.get_file_type() ||
                                                                a.get_file_type()==b.get_file_type() &&
                                                                a.get_name()<b.get_name(); });
                for(let i=filelist.length-1; i >= 0;i--) {
                    let info = filelist[i];
                    try {
                        filelist[i].get_name();
                    } catch(e) {
                        continue;
                    }
                    let fileType = info.get_file_type();
                    let name = info.get_name();
                    let icon = null;
                    if(name && name.search(skipReg) != -1)
                        continue;
                    if (fileType == Gio.FileType.DIRECTORY) {
                        try {
                            let iconDirectory = Gio.file_new_for_path(this.dirname+"/"+name+"/.directory");
                            let res,content,data,icon_name;
                            [res,content,data] = iconDirectory.load_contents(null);
                            if ( (res=regIcon.exec(content)) != null ) {
                                icon_name = res[2];
                            }
                            let icon_names = [];
                            icon_names.push(icon_name);
                            icon_names.push(icon_name.replace("downloads","download"));
                            icon_names.push(icon_name.replace("sound","music"));
                            icon_names.push(icon_name.replace("image","pictures"));
                            icon_names.push(icon_name.replace("html","publicshare"));
                            icon_names.push(icon_name.replace("txt","templates"));
                            icon_names.push(icon_name.replace("video","videos"));
                            icon_names.push("stock_folder");

                            icon = new Gio.ThemedIcon({names:icon_names,use_default_fallbacks:false});
                        } catch (e) {
                            icon = Gio.content_type_get_icon(info.get_content_type());
                        }
                    } else {
                        icon = Gio.content_type_get_icon(info.get_content_type());
                    }

                    let empty_item = new PopupMenuItem(name,
                                                        icon,
                                                        Lang.bind(this, function() {
                        let ff = Gio.file_new_for_path(this.dirname+"/"+name);
                        this._focusDesktopActor();
                        Gio.app_info_launch_default_for_uri(ff.get_uri(),null);
                    }));
                    this.submenu.menu.addMenuItem(empty_item);
                }

                PopupMenu.PopupMenu.prototype.open.call(this.menu, animate);
                this.submenu.menu.open(true);
                this.submenu.actor.height = 0;
                this.submenu.actor.opacity = 0;
            }));
        } catch (e) {
            global.logError('' + e);
            print(''+e);
            return;
        }
    }
};

function QLContainer() {
    this._init();
}

QLContainer.prototype = {
    _init: function() {
    },

    addItem: function(filename) {
        this.removeItem(filename);

        if (filename.indexOf('.desktop') < 0)
            return;

        let pathname = qlPath + '/' + filename;
        let fileObj = Gio.file_new_for_path(pathname);
        let type = fileObj.query_file_type(Gio.FileQueryInfoFlags.NONE, null);
        if (type != 1)
            return;

        let appinfo = new Gio.DesktopAppInfo.new_from_filename(pathname);
        if (!appinfo)
            return;

        let item = new QLButton(appinfo);
        this.items[filename] = item;

        let itemkeys = [];
        for (let i in this.items)
            itemkeys.push(i);
        itemkeys.sort();
        let index = itemkeys.indexOf(filename);
        this.actor.insert_actor(item.actor, index);
        Main.panel._menus.addMenu(item.menu);
    },

    removeItem: function(filename) {
        if (!(filename in this.items))
            return;

        let item = this.items[filename];
        item.actor.destroy();
        delete this.items[filename];
    },

    enable: function() {
        this.items = new Object();

        this.actor = new St.BoxLayout({ name: 'qlContainer',
                                        style_class: 'qlcontainer' });

        let qlPathObj = Gio.file_new_for_path(qlPath);

        this.qlPathMon = qlPathObj.monitor_directory(Gio.FileMonitorFlags.NONE, null);
        this.qlPathMonSigId = this.qlPathMon.connect('changed',
            Lang.bind(this, function (monitor, f, otherf, event) {
                if (event == 1)
                    this.addItem(f.get_basename())
                else if (event == 2)
                    this.removeItem(f.get_basename())
                else if (event == 3)
                    this.addItem(f.get_basename())
        }));

        FileUtils.listDirAsync(qlPathObj, Lang.bind(this, function(files) {
            for (let i = 0; i < files.length; i++)
                this.addItem(files[i].get_name())
        }));

        Main.panel._leftBox.insert_actor(this.actor, 2);
    },

    disable: function() {
        Main.panel._leftBox.remove_actor(this.actor);

        this.qlPathMon.disconnect(this.qlPathMonSigId);
        delete this.qlPathMonSigId;
        delete this.qlPathMon;

        for (let i in this.items) {
                this.removeItem(i);
        }

        delete this.actor;
        delete this.items;
    }
};

function init() {
    return new QLContainer();
}
