// Copyright (C) 2012 Calculate Ltd
// Licence: GPLv2+

const Clutter = imports.gi.Clutter;
const Lang = imports.lang;
const St = imports.gi.St;
const Shell = imports.gi.Shell;
const DateMenu = imports.ui.dateMenu;
const Calendar = imports.ui.calendar;
const Gio = imports.gi.Gio;
const Gdk = imports.gi.Gdk;

const Main = imports.ui.main;
const Mainloop = imports.mainloop;
const PopupMenu = imports.ui.popupMenu;
const uuid_appmenu = 'Applications_Menu@rmy.pobox.com';
const ExtensionSystem = imports.ui.extensionSystem;

let openitem = null;
let changeExtConnectId = null;
let oldBuildMenu = null;

function _newBuildMenu() {
    const AppMenuExtension = imports.ui.extensionSystem.extensions[uuid_appmenu].extension;
    let sections = this._buildSections();
    for ( let i=0; i<sections.length; ++i ) {
        let section = sections[i];
        let submenu = new PopupMenu.PopupSubMenuMenuItem(section.name);
        submenu.menu.open = Lang.bind(submenu.menu,function(animate) {
            if(openitem) {
                if(openitem.isOpen) {
                    let it=openitem;
                    if(it != this)
                        it.close();
                }
            }
            openitem = this;
                PopupMenu.PopupSubMenu.prototype.open.call(this,animate);
        });
        this.menu.addMenuItem(submenu);

        for ( let j=0; j<section.apps.length; ++j ) {
            let app = section.apps[j];
            let menuItem = new AppMenuExtension.ApplicationMenuItem(app);

            submenu.menu.addMenuItem(menuItem, j);
        }
    }
}

function tryEnablePlugin() {
    let appMenu = ExtensionSystem.extensionStateObjs[uuid_appmenu]
    if(appMenu) {
        oldBuildMenu = appMenu._buildMenu;
        appMenu._buildMenu = Lang.bind(appMenu,_newBuildMenu);
        appMenu._rebuildMenu();
    }
}

function extensionStateChanged(o,meta) {
    if(meta && meta.uuid == uuid_appmenu &&
        (meta.state == ExtensionSystem.ExtensionState.ENABLED ||
         meta.state == ExtensionSystem.ExtensionState.INITIALIZED)) {
        tryEnablePlugin();
    }
}

function init() {
}

function enable() {
    tryEnablePlugin();
    changeExtConnectId = ExtensionSystem.connect('extension-state-changed',extensionStateChanged);
    extensionStateChanged(null);
}

function disable() {
    if (changeExtConnectId) {
         ExtensionSystem.disconnect(changeExtConnectId);
         changeExtConnectId = null;
    }
    let appMenu = ExtensionSystem.extensionStateObjs[uuid_appmenu]
    if(appMenu && oldBuildMenu) {
        appMenu._buildMenu = oldBuildMenu;
        appMenu._rebuildMenu();
    }
    oldBuildMenu = null;
}
