// Copyright (C) 2011 Mir Calculate
// Licence: GPLv2+

const Lang = imports.lang;
const DBus = imports.dbus;
const ScreenSaver = imports.misc.screenSaver;

const UPowerIface = {
    name: 'org.freedesktop.UPower',
    signals: [{name: 'Changed',inSignature: ''}],
    properties: [{name: 'LidIsClosed',
                  signature: 'b',
                  access: 'readonly'}]
};

function UPowerProxy() {
    this._init();
}

UPowerProxy.prototype = {
    _init: function() {
        DBus.system.proxifyObject(this,
                                  'org.freedesktop.UPower',
                                  '/org/freedesktop/UPower');
        this._screenSaverProxy = new ScreenSaver.ScreenSaverProxy();
        this.connect('Changed',Lang.bind(this,this._onChanged));
        this.lock = false;
    },

    _onChanged: function(object) {
        this.GetRemote('LidIsClosed',Lang.bind(this,function(flag,error) {
            if(flag && this.lock) {
                this._screenSaverProxy.LockRemote();
            }
        }));
    },

    enable: function() {
        this.lock = true;
    },

    disable: function() {
        this.lock = false;
    }
};
DBus.proxifyPrototype(UPowerProxy.prototype, UPowerIface);

function init(extensionMeta) {
    return new UPowerProxy();
}
