// Copyright (C) 2011 R M Yorston
// Licence: GPLv2+
// Code of Workspace panel get from Frippery Bottom Panel

const Clutter = imports.gi.Clutter;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Lang = imports.lang;
const Mainloop = imports.mainloop;
const Meta = imports.gi.Meta;
const Pango = imports.gi.Pango;
const Shell = imports.gi.Shell;
const Signals = imports.signals;
const St = imports.gi.St;
const WorkspaceThumbnail = imports.ui.workspaceThumbnail;
const DND = imports.ui.dnd;

const Main = imports.ui.main;
const ModalDialog = imports.ui.modalDialog;

const _f = imports.gettext.domain('frippery-bottom-panel').gettext;

let nrows = 1;

function get_ncols() {
    let ncols = Math.floor(global.screen.n_workspaces/nrows);
    if ( global.screen.n_workspaces%nrows != 0 )
       ++ncols

    return ncols;
}

function NewWindowClone(realWindow) {
    this._init(realWindow);
}

NewWindowClone.prototype = {
    __proto__: WorkspaceThumbnail.WindowClone.prototype,

    _init : function(realWindow) {
        this.actor = new Clutter.Clone({ source: realWindow.get_texture(),
                                         reactive: true });
        this.actor._delegate = this;
        this.realWindow = realWindow;
        this.metaWindow = realWindow.meta_window;

        this._focusChangedId = this.metaWindow.connect('focus', Lang.bind(this, function(num) {
                                                            this.actor.raise_top();
        }));
        this._wsChangedId = this.metaWindow.connect('workspace-changed', Lang.bind(this, function() {
                                                            this.actor.raise_top();
                                                            this.metaWindow.raise();
                                                            let tracker = Shell.WindowTracker.get_default();
                                                            let focusedApp = tracker.focus_app;
                                                            global.display.focus_window.raise();
        }));
        this._positionChangedId = this.realWindow.connect('position-changed',
                                                          Lang.bind(this, this._onPositionChangedWithDelay));
        this._realWindowDestroyedId = this.realWindow.connect('destroy',
                                                              Lang.bind(this, this._disconnectRealWindowSignals));
        this._onEventId = null;
        this._onPositionChanged();

        this.actor.connect('button-release-event',
                           Lang.bind(this, this._onButtonRelease));

        this.actor.connect('destroy', Lang.bind(this, this._onDestroy));

        this._draggable = DND.makeDraggable(this.actor,
                                            { restoreOnSuccess: true,
                                              dragActorMaxSize: WorkspaceThumbnail.Workspace.WINDOW_DND_SIZE,
                                              dragActorOpacity: WorkspaceThumbnail.Workspace.DRAGGING_WINDOW_OPACITY });
        this._draggable.connect('drag-begin', Lang.bind(this, this._onDragBegin));
        this._draggable.connect('drag-end', Lang.bind(this, this._onDragEnd));
        this.inDrag = false;
        this._countDelay = 0;
        this._timeout = null;
    },

    _disconnectRealWindowSignals: function() {
        WorkspaceThumbnail.WindowClone.prototype._disconnectRealWindowSignals.call(this);
        if (this._focusChangedId != 0) {
            this.metaWindow.disconnect(this._focusChangedId);
            this._focusChangedId = 0;
        }
        if (this._wsChangedId != 0) {
            this.metaWindow.disconnect(this._wsChangedId);
            this._focusChangedId = 0;
        }
    },

    _onPositionChangedWithDelay : function() {
        if(this._timeout) {
            Mainloop.source_remove(this._timeout);
        }
        if(++this._countDelay > 5) {
            this._onPositionChanged();
            this._countDelay = 0;
        }
        Mainloop.timeout_add_seconds(2, Lang.bind(this,function() {
            this._onPositionChanged();
            this._timeout = null;
            this._countDelay = 0;
        }));
    }
}

function WorkspaceDialog() {
    this._init();
}

WorkspaceDialog.prototype = {
    __proto__: ModalDialog.ModalDialog.prototype,

    _init: function() {
        ModalDialog.ModalDialog.prototype._init.call(this, { styleClass: 'workspace-dialog' });

        let label = new St.Label({ style_class: 'workspace-dialog-label',
                                   text: _f('Number of workspaces') });
        this.contentLayout.add(label, { y_align: St.Align.START });

        let entry = new St.Entry({ style_class: 'workspace-dialog-entry' });

        this._workspaceEntry = entry.clutter_text;
        this.contentLayout.add(entry, { y_align: St.Align.START });
        this.setInitialKeyFocus(this._workspaceEntry);

        this._workspaceEntry.connect('key-press-event',
                Lang.bind(this, this._onKeyPress));

        label = new St.Label({ style_class: 'workspace-dialog-label',
                                   text: _f('Rows in workspace switcher') });
        this.contentLayout.add(label, { y_align: St.Align.START });

        entry = new St.Entry({ style_class: 'workspace-dialog-entry' });

        this._rowEntry = entry.clutter_text;
        this.contentLayout.add(entry, { y_align: St.Align.START });

        this._rowEntry.connect('key-press-event',
                Lang.bind(this, this._onKeyPress));

    },

    open: function() {
        this._workspaceEntry.set_text(''+global.screen.n_workspaces);
        this._rowEntry.set_text(''+nrows);

        ModalDialog.ModalDialog.prototype.open.call(this);
    },

    _onKeyPress: function(actor, event) {
        let symbol = event.get_key_symbol();
        if (symbol == Clutter.Return || symbol == Clutter.KP_Enter) {
            let num = parseInt(this._workspaceEntry.get_text());
            if ( !isNaN(num) && num >= 2 && num <= 32 ) {
                let old_num = global.screen.n_workspaces;
                if ( num > old_num ) {
                    for ( let i=old_num; i<num; ++i ) {
                        global.screen.append_new_workspace(false,
                                global.get_current_time());
                    }
                }
                else if ( num < old_num ) {
                    for ( let i=old_num-1; i>=num; --i ) {
                        let ws = global.screen.get_workspace_by_index(i);
                        global.screen.remove_workspace(ws,
                                global.get_current_time());
                    }
                }
            }

            let rows = parseInt(this._rowEntry.get_text());
            if ( !isNaN(rows) && rows > 0 && rows < 6 && rows != nrows ) {
                nrows = rows;
                bottomPanel.workspaceSwitcher._createButtons();

                let rowFilePath = GLib.get_home_dir() + '/.frippery_rows';
                let rowFile = Gio.file_new_for_path(rowFilePath);
                rowFile.replace_contents(''+rows+'\n', null, false, 0, null);
            }

            this.close();
            return true;
        }
        else if (symbol == Clutter.Escape) {
            this.close();
            return true;
        }
        else if (symbol == Clutter.Tab) {
            if ( actor == this._rowEntry ) {
                global.stage.set_key_focus(this._workspaceEntry);
            }
            else {
                global.stage.set_key_focus(this._rowEntry);
            }
            return true;
        }
        else if (symbol == Clutter.Up && actor == this._rowEntry) {
            global.stage.set_key_focus(this._workspaceEntry);
            return true;
        }
        else if (symbol == Clutter.Down && actor == this._workspaceEntry) {
            global.stage.set_key_focus(this._rowEntry);
            return true;
        }

        return false;
    }
};
Signals.addSignalMethods(WorkspaceDialog.prototype);

function WorkspaceSwitcher() {
    this._init();
}

WorkspaceSwitcher.prototype = {
    _init: function() {
        this.actor = new St.BoxLayout({ name: 'workspaceSwitcher',
                                        style_class: 'workspace-switcher',
                                        reactive: true });
        this.actor.connect('button-release-event', this._showDialog);
        this.actor.connect('scroll-event', this._onScroll);
        this.actor._delegate = this;
        this.button = [];
        this._createButtons();

        global.screen.connect('notify::n-workspaces',
                                Lang.bind(this, this._createButtons));
        global.window_manager.connect('switch-workspace',
                                Lang.bind(this, this._updateButtons));
    },

    _createButtons: function() {
        this.actor.destroy_children();
        this.button = [];

        let ncols = get_ncols();
        let active = global.screen.get_active_workspace_index();
        let row = Math.floor(active/ncols);

        this.row_indicator = null;
        if ( nrows > 1 ) {
            this.row_indicator = new St.DrawingArea({ reactive: true,
                                    style_class: 'workspace-row-indicator' });
            this.row_indicator.connect('repaint', Lang.bind(this, this._draw));
            this.row_indicator.connect('button-press-event', Lang.bind(this, this._rowButtonPress));
            this.row_indicator.connect('scroll-event', Lang.bind(this, this._rowScroll));
            this.actor.add(this.row_indicator);
        }

        let monitor = Main.layoutManager.primaryMonitor;
        let monWidth = monitor.width;
        let monHeight = monitor.height;
        let panelHeight = Main.panel.actor.height;
        let workspaceHeight = panelHeight;
        let my_scale = (workspaceHeight-1)/monHeight;
        let workspaceWidth = monWidth * (workspaceHeight)/monHeight;
        this.actor.height = panelHeight;
        for ( let i=0; i<ncols; ++i ) {
            let index = row*ncols + i;

            this.button[i] = new St.Button({ name: 'workspaceButton',
                                     style_class: 'workspace-button',
                                     reactive: true });
            let metaWorkspace = global.screen.get_workspace_by_index(index);
            let text = '';
            if ( index == active ) {
                //Text = '-' + (index+1).toString() + '-';
                this.button[i].add_style_pseudo_class('outlined');
            }
            else if ( index < global.screen.n_workspaces ) {
                //text = (index+1).toString();
            }
            this.actor.add(this.button[i]);
            this.button[i].set_size(workspaceWidth,workspaceHeight);
            let wt = new WorkspaceThumbnail.WorkspaceThumbnail(metaWorkspace);
            wt._addWindowClone = Lang.bind(wt,function(win) {
                let clone = new NewWindowClone(win);
                clone.connect('selected',
                       Lang.bind(this, this._activate));
                clone.connect('drag-begin',
                       Lang.bind(this, function(clone) {
                            Main.overview.beginWindowDrag();
                }));
                clone.connect('drag-end',
                      Lang.bind(this, function(clone) {
                        Main.overview.endWindowDrag();
                }));
                this._contents.add_actor(clone.actor);

                if (this._windows.length == 0)
                    clone.setStackAbove(this._background);
                else
                    clone.setStackAbove(this._windows[this._windows.length - 1].actor);

                this._windows.push(clone);

                return clone;
            });
            wt._contents.set_scale(my_scale,my_scale);
            this.button[i].set_child(wt.actor);
            this.button[i].connect('clicked', Lang.bind(this, this._onClicked));
        }

        global.screen.override_workspace_layout(Meta.ScreenCorner.TOPLEFT,
               false, nrows, ncols);

    },

    _updateButtons: function() {
        let ncols = get_ncols();
        let active = global.screen.get_active_workspace_index();
        let row = Math.floor(active/ncols);

        for ( let i=0; i<this.button.length; ++i ) {
            let index = row*ncols + i;

            if ( index == active ) {
                //this.button[i].get_child().set_text('-' + (index+1).toString() + '-');
                this.button[i].add_style_pseudo_class('outlined');
            }
            else if ( index < global.screen.n_workspaces ) {
                //this.button[i].get_child().set_text((index+1).toString());
                this.button[i].remove_style_pseudo_class('outlined');
            }
            else {
                //this.button[i].get_child().set_text('');
                this.button[i].remove_style_pseudo_class('outlined');
            }
        }

        if ( this.row_indicator ) {
            this.row_indicator.queue_repaint();
        }
    },

    _showDialog: function(actor, event) {
        let button = event.get_button();
        if ( button == 3 ) {
            if ( this._workspaceDialog == null ) {
                this._workspaceDialog = new WorkspaceDialog();
            }
            this._workspaceDialog.open();
            return true;
        }
        return false;
    },

    _onClicked: function(btn) {
        let i;
        for ( i=0; i<this.button.length; ++i ) {
            if ( this.button[i] == btn ) {
                break;
            }
        }

        let ncols = get_ncols();
        let active = global.screen.get_active_workspace_index();
        let row = Math.floor(active/ncols);
        let index = row*ncols + i;

        if ( index >= 0 && index < global.screen.n_workspaces ) {
            let metaWorkspace = global.screen.get_workspace_by_index(index);
            metaWorkspace.activate(global.get_current_time());
        }

        return true;
    },

    _onScroll: function(actor, event) {
        let direction = event.get_scroll_direction();
        let ncols = get_ncols();
        let active = global.screen.get_active_workspace_index();
        let index = global.screen.n_workspaces;

        if ( direction == Clutter.ScrollDirection.DOWN ) {
            if ( active%ncols > 0 ) {
                index = active-1;
            }
        }
        if ( direction == Clutter.ScrollDirection.UP ) {
            if ( active < global.screen.n_workspaces-1 &&
                         active%ncols != ncols-1 ) {
                index = active+1;
            }
        }

        if ( index >= 0 && index < global.screen.n_workspaces ) {
            let metaWorkspace = global.screen.get_workspace_by_index(index);
            metaWorkspace.activate(global.get_current_time());
        }

        return true;
    },

    _rowButtonPress: function(actor, event) {
        if ( event.get_button() != 1 ) {
            return false;
        }

        let ncols = get_ncols();
        let active = global.screen.get_active_workspace_index();
        let row = Math.floor(active/ncols);

        let [x, y] = event.get_coords();
        let [wx, wy] = actor.get_transformed_position();
        let [w, h] = actor.get_size();
        y -= wy;

        let new_row = Math.floor(nrows*y/h);
        let index = global.screen.n_workspaces;
        if ( new_row != row ) {
            index = new_row*ncols + active%ncols;
        }

        if ( index >= 0 && index < global.screen.n_workspaces ) {
            let metaWorkspace = global.screen.get_workspace_by_index(index);
            metaWorkspace.activate(global.get_current_time());
        }

        return true;
    },

    _rowScroll: function(actor, event) {
        let direction = event.get_scroll_direction();
        let ncols = get_ncols();
        let active = global.screen.get_active_workspace_index();
        let row = Math.floor(active/ncols);

        let index = global.screen.n_workspaces;
        if ( direction == Clutter.ScrollDirection.DOWN ) {
            index = (row+1)*ncols + active%ncols;
        }
        if ( direction == Clutter.ScrollDirection.UP ) {
            index = (row-1)*ncols + active%ncols;
        }

        if ( index >= 0 && index < global.screen.n_workspaces ) {
            let metaWorkspace = global.screen.get_workspace_by_index(index);
            metaWorkspace.activate(global.get_current_time());
        }

        return true;
    },

    _draw: function(area) {
        let [width, height] = area.get_surface_size();
        let themeNode = this.row_indicator.get_theme_node();
        let cr = area.get_context();

        let active_color = themeNode.get_color('-active-color');
        let inactive_color = themeNode.get_color('-inactive-color');

        let ncols = get_ncols();
        let active = global.screen.get_active_workspace_index();
        let row = Math.floor(active/ncols);

        for ( let i=0; i<nrows; ++i ) {
            let y = (i+1)*height/(nrows+1);
            cr.moveTo(0, y);
            cr.lineTo(width, y);
            let color = row == i ? active_color : inactive_color;
            Clutter.cairo_set_source_color(cr, color);
            cr.setLineWidth(2.0);
            cr.stroke();
        }
    }
};

let oldMessageTrayAlign,wSwitcher = null;

function init() {
	oldMessageTrayAlign = Main.messageTray._notificationBin.x_align 
}

function enable() {
	Main.messageTray._notificationBin.x_align = St.Align.END;
	Main.messageTray._notificationBin.raise_top();
	Main.messageTray._notificationBin.add_style_class_name('notification-box');
        wSwitcher = new WorkspaceSwitcher();
	Main.panel._leftBox.insert_actor(wSwitcher.actor, 1);
        Main.overview._viewSelector._tabBox.hide();
	Main.overview._workspacesDisplay._thumbnailsBox.actor.width = 0;
	Main.overview._workspacesDisplay.actor.hide();
}

function disable() {
	Main.messageTray._notificationBin.x_align = oldMessageTrayAlign;
	wSwitcher.actor.destroy();
}
